using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;
using System.ComponentModel;

namespace CustomControls.TemplatedControls
{
	/// <summary>
	/// Templated Control  6
	/// </summary>
	[
	ParseChildren(true)
	]
	public class TemplatedControl6 : WebControl, INamingContainer
	{
		#region  
		private static readonly object EventSelectedIndexChanged = new object();
		private static readonly object EventItemCreated = new object();
		private static readonly object EventItemDataBound = new object();
		private static readonly object EventItemCommand = new object();
		#endregion

		private ITemplate itemTemplate = null;
		private ITemplate alternatingTemplate = null;
		private IEnumerable dataSource = null;

		#region Ӽ
		public IEnumerable DataSource 
		{
			get {return dataSource;}
			set {dataSource = value;}
		}

		[TemplateContainer(typeof(TemplatedControlItem))]
		public ITemplate ItemTemplate 
		{
			get {return itemTemplate;}
			set {itemTemplate = value;}
		}

		[TemplateContainer(typeof(TemplatedControlItem))]
		public ITemplate AlternatingTemplate 
		{
			get {return alternatingTemplate;}
			set {alternatingTemplate = value;}
		}

		public virtual int SelectedIndex 
		{
			get 
			{
				object o = ViewState["SelectedIndex"];
				if (o != null)
					return(int)o;
				else
					return -1;
			}
			set 
			{
				if (value < -1) 
					throw new ArgumentOutOfRangeException();

				ViewState["SelectedIndex"] = value;
			}
		}

		private int ItemCount
		{
			get 
			{
				object o = ViewState["itemCount"];
				if(o == null)
					return 0;
				else
					return (int)o;
			}
			set {ViewState["itemCount"] = value;}
		}
		#endregion

		#region ̺Ʈ
		protected override bool OnBubbleEvent(object source, EventArgs e) 
		{
			if (e is TemplatedControlCommandEventArgs) 
			{
				TemplatedControlCommandEventArgs ce = (TemplatedControlCommandEventArgs)e;

				OnItemCommand(ce);

				if (String.Compare(ce.CommandName, "Select", true) == 0) 
				{
					SelectedIndex = ce.Item.ItemIndex;
					OnSelectedIndexChanged(EventArgs.Empty);
				}

				// ̺Ʈ ֻ Ʈѿ óϵ Ѵ.
				return true;
			}

			// ̺Ʈ ܿ  ó Ѵ
			return false;
		}

		protected virtual void OnItemCommand(TemplatedControlCommandEventArgs e) 
		{
			TemplatedControlCommandEventHandler onItemCommandHandler = (TemplatedControlCommandEventHandler)Events[EventItemCommand];
			if (onItemCommandHandler != null) onItemCommandHandler(this, e);
		}

		protected virtual void OnItemCreated(TemplatedControlItemEventArgs e) 
		{
			TemplatedControlItemEventHandler onItemCreatedHandler = (TemplatedControlItemEventHandler)Events[EventItemCreated];
			if (onItemCreatedHandler != null) onItemCreatedHandler(this, e);
		}

		protected virtual void OnItemDataBound(TemplatedControlItemEventArgs e) 
		{
			TemplatedControlItemEventHandler onItemDataBoundHandler = (TemplatedControlItemEventHandler)Events[EventItemDataBound];
			if (onItemDataBoundHandler != null) onItemDataBoundHandler(this, e);
		}

		protected virtual void OnSelectedIndexChanged(EventArgs e) 
		{
			EventHandler handler = (EventHandler)Events[EventSelectedIndexChanged];
			if (handler != null) handler(this, e);
		}

		public event TemplatedControlCommandEventHandler ItemCommand 
		{
			add 
			{
				Events.AddHandler(EventItemCommand, value);
			}
			remove 
			{
				Events.RemoveHandler(EventItemCommand, value);
			}
		}

		public event TemplatedControlItemEventHandler ItemCreated 
		{
			add 
			{
				Events.AddHandler(EventItemCreated, value);
			}
			remove 
			{
				Events.RemoveHandler(EventItemCreated, value);
			}
		}

		[
		Category("Behavior"),
		Description("Raised when an item is data-bound.")
		]
		public event TemplatedControlItemEventHandler ItemDataBound 
		{
			add 
			{
				Events.AddHandler(EventItemDataBound, value);
			}
			remove 
			{
				Events.RemoveHandler(EventItemDataBound, value);
			}
		}

		public event EventHandler SelectedIndexChanged 
		{
			add 
			{
				Events.AddHandler(EventSelectedIndexChanged, value);
			}
			remove 
			{
				Events.RemoveHandler(EventSelectedIndexChanged, value);
			}
		}
#endregion

		protected override void CreateChildControls()
		{
			Controls.Clear();

			if(ItemCount != 0)
				CreateControlHierarchy(false);
		}

		public override void DataBind()
		{
			//   ε ϱ   ű
			base.DataBind();

			Controls.Clear();

			if (dataSource != null) 
			{
				if(HasChildViewState)
					ClearChildViewState();

				CreateControlHierarchy(true);

				ChildControlsCreated = true;
			}
		}

		private void CreateControlHierarchy(bool useDataSource) 
		{
			IEnumerable datSrc = null;

			if (useDataSource) 
				datSrc = this.dataSource;
			else 
				datSrc = new DummyDataSource(ItemCount);

			if (datSrc != null) 
			{
				int count = 0;
				foreach(object dataItem in datSrc)
				{
					ListItemType itemType;
					ITemplate template;
					
					if (count % 2 == 0 || alternatingTemplate == null)
					{
						itemType = ListItemType.Item;
						template = itemTemplate;
					}
					else
					{
						itemType = ListItemType.AlternatingItem;
						template = alternatingTemplate;
					}

					TemplatedControlItem item = new TemplatedControlItem(count, itemType, dataItem);
					TemplatedControlItemEventArgs e = new TemplatedControlItemEventArgs(item);

					OnItemCreated(e);
					template.InstantiateIn(item);

					if (useDataSource)
					{
						item.DataBind();
						OnItemDataBound(e);
					}
					Controls.Add(item);
					count++;
				}

				if (useDataSource) 
					ItemCount = count;
			}
		}

		internal class DummyDataSource : IEnumerable
		{
			private int itemCount;

			public DummyDataSource(int itemCount) 
			{
				this.itemCount = itemCount;
			}

			public IEnumerator GetEnumerator() 
			{
				return new DummyDataSourceEnumerator(itemCount);
			}

			private class DummyDataSourceEnumerator : IEnumerator 
			{
				private int count;
				private int index;

				public DummyDataSourceEnumerator(int count) 
				{
					this.count = count;
					this.index = -1;
				}

				public object Current 
				{
					get {return null;}
				}

				public bool MoveNext() 
				{
					index++;
					return index < count;
				}

				public void Reset() 
				{
					this.index = -1;
				}
			}
		}
	}

	public class TemplatedControlItem : Control, INamingContainer 
	{
		private int itemIndex;
		private ListItemType itemType;
		private object dataItem;

		public TemplatedControlItem(int itemIndex, ListItemType itemType, object dataItem) 
		{
			this.itemIndex = itemIndex;
			this.itemType = itemType;
			this.dataItem = dataItem;
		}

		public virtual object DataItem 
		{
			get {return dataItem;}
			set {dataItem = value;}
		}

		public virtual int ItemIndex 
		{
			get {return itemIndex;}
		}

		public virtual ListItemType ItemType 
		{
			get {return itemType;}
		}

		protected override bool OnBubbleEvent(object source, EventArgs e) 
		{
			if (e is CommandEventArgs) 
			{
				TemplatedControlCommandEventArgs args =
					new TemplatedControlCommandEventArgs(this, source, (CommandEventArgs)e);

				RaiseBubbleEvent(this, args);
				// ̺Ʈ ֻ Ʈѿ óϵ Ѵ.
				return true;
			}
			// ̺Ʈ ܿ  ó Ѵ
			return false;
		}
	}

	//̺Ʈ    ̺Ʈ Ѱ .
	public sealed class TemplatedControlCommandEventArgs : CommandEventArgs 
	{
		private TemplatedControlItem item;
		private object commandSource;

		public TemplatedControlCommandEventArgs (TemplatedControlItem item, object commandSource, CommandEventArgs originalArgs) :
			base(originalArgs) 
		{
			this.item = item;
			this.commandSource = commandSource;
		}

		public TemplatedControlItem Item 
		{
			get {return item;}
		}

		public object CommandSource 
		{
			get {return commandSource;}
		}
	}

	public delegate void TemplatedControlCommandEventHandler(object source, TemplatedControlCommandEventArgs  e);

	//OnItemCreate OnItemDataBound ޼ҵ .
	public sealed class TemplatedControlItemEventArgs : EventArgs 
	{

		private TemplatedControlItem item;

		public TemplatedControlItemEventArgs(TemplatedControlItem item) 
		{
			this.item = item;
		}

		public TemplatedControlItem Item 
		{
			get 
			{
				return item;
			}
		}
	}

	public delegate void TemplatedControlItemEventHandler(object sender, TemplatedControlItemEventArgs e);
}
